@@ -19,12 +19,7 @@
 
 #include <algorithm>
 #include <cerrno>
-#include <climits>
-#include <cstring>
-#include <string>
-#include <vector>
-#include <pthread.h>	// for tarlz.h
-#include <stdint.h>
+#include <stdint.h>	// for lzlib.h
 #include <unistd.h>
 #include <lzlib.h>
 
@@ -87,7 +82,7 @@ int Archive_reader_base::parse_records( Extended & extended,
   if( edsize <= 0 || edsize >= 1LL << 33 || bufsize >= INT_MAX )
     return 1;					// overflow or no extended data
   if( !rbuf.resize( bufsize ) ) return 1;	// extended records buffer
-  int retval = read( (uint8_t *)rbuf(), bufsize );
+  int retval = read( rbuf.u8(), bufsize );
   if( retval == 0 && !extended.parse( rbuf(), edsize, permissive ) )
     retval = 2;
   return retval;
