@@ -17,14 +17,8 @@
 
 #define _FILE_OFFSET_BITS 64
 
-#include <climits>
 #include <cstdlib>
-#include <cstring>
-#include <string>
-#include <vector>
 #include <fnmatch.h>
-#include <pthread.h>	// for tarlz.h
-#include <stdint.h>
 
 #include "tarlz.h"
 
