@@ -18,14 +18,8 @@
 #define _FILE_OFFSET_BITS 64
 
 #include <cctype>
-#include <climits>
 #include <cstdio>
 #include <cstdlib>
-#include <cstring>
-#include <string>
-#include <vector>
-#include <pthread.h>	// for tarlz.h
-#include <stdint.h>
 
 #include "tarlz.h"
 
@@ -173,7 +167,7 @@ long long Extended::format_block( Resizable_buffer & rbuf ) const
   if( edsize_ <= 0 ) return 0;			// no extended data
   if( edsize_ >= 1LL << 33 ) return -1;		// too much extended data
   if( !rbuf.resize( bufsize ) ) return -1;	// extended block buffer
-  uint8_t * const header = (uint8_t *)rbuf();	// extended header
+  uint8_t * const header = rbuf.u8();		// extended header
   char * const buf = rbuf() + header_size;	// extended records
   init_tar_header( header );
   header[typeflag_o] = tf_extended;		// fill only required fields
